// Andreas 'BlackHC' Kirsch 2009
// TODO: add shutdown code and delete the buffer and shader objects

#include <string>

#include <GL/glew.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <glut.h>

#include <stdio.h>
#include <assert.h>

GLuint program;
GLuint buffer;

GLfloat vertices[1] = {
	0
};

void checkGLError(void) {
	GLenum error = glGetError();
	if( error != GL_NO_ERROR ) {
		printf( "%s\n", gluErrorString(error) );
	}
}

void initBuffer() {
	glGenBuffers( 1, &buffer );
	glBindBuffer(GL_ARRAY_BUFFER, buffer);
	glBufferData(GL_ARRAY_BUFFER, sizeof(vertices), vertices, GL_STATIC_DRAW);
	checkGLError();
}

GLuint setupShader(GLuint shaderType, const char *source) {
	GLuint shader = glCreateShader(shaderType);
	assert( shader );
	glShaderSource(shader, 1, &source, NULL );
	glCompileShader(shader);

	checkGLError();

	return shader;	
}

std::string loadFile(const char *filename) {
	FILE *file = fopen(filename, "rb");
	fseek( file, 0, SEEK_END );
	int size = ftell( file );
	fseek(file, 0, SEEK_SET );

	char *data = new char[size+1];
	data[size] = 0;
	fread( data, size, 1, file );
	std::string buffer = data;

	delete[] data;
	fclose( file );

	return buffer;
}

void initShaders(void) {
	program = glCreateProgram();

	GLuint vertexShader = setupShader( GL_VERTEX_SHADER, loadFile( "vertex.shader" ).c_str() );
	GLuint fragmentShader = setupShader( GL_FRAGMENT_SHADER, loadFile( "fragment.shader" ).c_str() );
	
	glAttachShader( program, vertexShader );
	glAttachShader( program, fragmentShader );

	glLinkProgram(program);

	GLint infoSize;
	glGetProgramiv(program, GL_INFO_LOG_LENGTH, &infoSize);

	char *message = new char[infoSize];
	glGetProgramInfoLog(program, infoSize, NULL, message);
	printf( "%s\n", message );
	delete[] message;

	checkGLError();

	glUseProgram(program);
}

/*
unsigned packFloat( float v ) {
	return ((v + 1) / 2 * 255) & 255;
}

unsigned packData( float x, float y, float z ) {
	return packFloat( x ) + (packFloat( y ) << 8) + (packFloat( z ) << 16);
}*/

#define packFloat(v)	(int(((v) + 1.0) / 2 * 255) & 255)
#define packVertex(x,y,z) (packFloat( x ) + (packFloat( y ) << 8) + (packFloat( z ) << 16))

void display(void) {
	glClearColor(0,0,0,0);
	glClear(GL_COLOR_BUFFER_BIT);

	glVertexPointer(3, GL_FLOAT, 0, 0);
	glEnableClientState(GL_VERTEX_ARRAY);

	unsigned indices[] = {
		packVertex( 0.0, 0.0, -1.0 ), packVertex( 1.0, 0.0, -1.0 ), packVertex( 1.0, 1.0, -1.0 ),
		packVertex( 0.0, 0.0, -1.0 ), packVertex( -1.0, 0.0, -1.0 ), packVertex( -1.0, -1.0, -1.0 )
	};

	glDrawElements( GL_TRIANGLES, sizeof( indices ) / sizeof( *indices ), GL_UNSIGNED_INT, indices );

	checkGLError();
	
	glFinish();
}

void GLUTCALLBACK reshape(int width, int height) {
	glViewport(0,0, width, height);
}

int main(int argc, char * argv[]) {
	glutInit(&argc, argv);
	glutInitDisplayMode( GLUT_RGBA );

	glutCreateWindow("GLEW Test");
	glutDisplayFunc( display );
	glutReshapeFunc(reshape);

	GLenum err = glewInit();
	if (GLEW_OK != err)
	{
		/* Problem: glewInit failed, something is seriously wrong. */
		printf("Error: %s\n", glewGetErrorString(err));
	}
	printf("Status: Using GLEW %s\n", glewGetString(GLEW_VERSION));
	
	initShaders();
	initBuffer();

	glutMainLoop();
	return 0;
}
	